// -*- mode: c++ -*-
#ifndef _IMI_OPTION_KEYS_H
#define _IMI_OPTION_KEYS_H

/**
 * these strings are used as names of instances of COptionEvent
 */
#define SYSTEM_DATA_DIR                 "General/DataDir"
#define USER_DATA_DIR                   "General/UserDataDir"
#define PINYIN_SCHEME                   "Pinyin/Scheme"
#define PINYIN_PUNCTMAPPING_ENABLED     "General/PunctMapping/Enabled"
#define PINYIN_PUNCTMAPPING_MAPPINGS    "General/PunctMapping/Mappings"
#define CONFIG_GENERAL_CHARSET_LEVEL    "General/Charset"
#define QUANPIN_FUZZY_ENABLED           "QuanPin/Fuzzy/Enabled"
#define QUANPIN_FUZZY_PINYINS           "QuanPin/Fuzzy/Pinyins"
#define QUANPIN_AUTOCORRECTION_ENABLED  "QuanPin/AutoCorrection/Enabled"
#define QUANPIN_AUTOCORRECTION_PINYINS  "QuanPin/AutoCorrection/Pinyins"
#define SHUANGPIN_TYPE                  "Pinyin/ShuangPinType"

#endif // _IMI_OPTION_KEYS_H

// -*- indent-tabs-mode: nil -*- vim:et:ts=4
